"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const backtesting_1 = require("backtesting");
const event_stream_1 = __importDefault(require("event-stream"));
const mongodb_1 = require("mongodb");
const odata_v4_server_1 = require("odata-v4-server");
const connect_1 = __importDefault(require("../connect"));
const Trade_1 = require("./Trade");
const collectionName = "backtest";
class Backtest {
    constructor(data) {
        Object.assign(this, data);
    }
    execute(result) {
        return __awaiter(this, void 0, void 0, function* () {
            const { _id: key } = result;
            // tslint:disable-next-line: variable-name
            const _id = new mongodb_1.ObjectID(key);
            const db = yield connect_1.default();
            const backtest = new Backtest(yield db.collection(collectionName).findOne({ _id }));
            const { asset, currency, exchange, period, begin, end, indicators, initialBalance, code } = backtest;
            const rs = backtesting_1.streamTrades({
                exchange,
                currency,
                asset,
                period,
                start: begin,
                end,
                indicators: JSON.parse(indicators),
                code,
                initialBalance
            });
            let finalBalance;
            const tradesCollection = yield db.collection("trade");
            yield tradesCollection.deleteMany({ parentId: _id });
            rs.pipe(event_stream_1.default.map((chunk, next) => {
                const doc = JSON.parse(chunk);
                doc.parentId = _id;
                finalBalance = doc.amount;
                tradesCollection.insertOne(doc, next);
            }));
            yield new Promise(resolve => {
                rs.on("end", resolve);
            });
            return db
                .collection(collectionName)
                .updateOne({ _id }, {
                $set: {
                    finalBalance: Math.abs(finalBalance)
                }
            })
                .then(r => r.modifiedCount);
        });
    }
}
__decorate([
    odata_v4_server_1.Edm.Key,
    odata_v4_server_1.Edm.Computed,
    odata_v4_server_1.Edm.String
], Backtest.prototype, "_id", void 0);
__decorate([
    odata_v4_server_1.Edm.String
], Backtest.prototype, "exchange", void 0);
__decorate([
    odata_v4_server_1.Edm.String
], Backtest.prototype, "currency", void 0);
__decorate([
    odata_v4_server_1.Edm.String
], Backtest.prototype, "asset", void 0);
__decorate([
    odata_v4_server_1.Edm.Double
], Backtest.prototype, "period", void 0);
__decorate([
    odata_v4_server_1.Edm.String
], Backtest.prototype, "begin", void 0);
__decorate([
    odata_v4_server_1.Edm.String
], Backtest.prototype, "end", void 0);
__decorate([
    odata_v4_server_1.Edm.String
], Backtest.prototype, "indicators", void 0);
__decorate([
    odata_v4_server_1.Edm.String
], Backtest.prototype, "code", void 0);
__decorate([
    odata_v4_server_1.Edm.Double
], Backtest.prototype, "initialBalance", void 0);
__decorate([
    odata_v4_server_1.Edm.Double
], Backtest.prototype, "finalBalance", void 0);
__decorate([
    odata_v4_server_1.Edm.Double
], Backtest.prototype, "profit", void 0);
__decorate([
    odata_v4_server_1.Edm.Collection(odata_v4_server_1.Edm.EntityType(odata_v4_server_1.Edm.ForwardRef(() => Trade_1.Trade)))
], Backtest.prototype, "Trades", void 0);
__decorate([
    odata_v4_server_1.Edm.Action,
    __param(0, odata_v4_server_1.odata.result)
], Backtest.prototype, "execute", null);
exports.Backtest = Backtest;
//# sourceMappingURL=Backtest.js.map